from wedsite.conf import settings

def wedsite_context(request):
	"""
	Automatically adds all wedsite data to the template context s.t. we don't
	need custom views to do so.
	"""
	return settings.WEDSITE_JSON


# -*- coding: utf-8 -*-

from django.utils.encoding import smart_str

from geopy import geocoders
from geopy.exc import GeocoderServiceError

from wedsite import settings


class Error(Exception):
    pass


def google_v3(address):
    """
    Given an address, return ``(computed_address, (latitude, longitude))``
    tuple using Google Geocoding API v3.

    """
    try:
        g = geocoders.GoogleV3(
            api_key=getattr(settings, 'EASY_MAPS_GOOGLE_MAPS_API_KEY', None))

        results = g.geocode(smart_str(address), exactly_one=False)
        if results is not None:
            return results[0]

        raise Error("No results found for '%s'" % address)
    except (UnboundLocalError, ValueError, GeocoderServiceError) as e:
        raise Error(e)



from django import forms
from django.contrib.auth.models import User
from django.contrib.auth import password_validation
from wedsite.models import RSVP, RSVPPerson
from django.forms import inlineformset_factory
import re
from wedsite.conf import settings

# Width of form fields
FORM_FIELD_WIDTH = '64'

class CreateUserForm(forms.Form):
    """
    Base form for creating a new user. will ask for a username and password,
    and will also get their first and last names and address
    """

    # First and Last Name
    first_name = forms.CharField(
        label="First Name",
        max_length=30,
        widget=forms.TextInput(
            attrs={'autofocus': True, 'size':FORM_FIELD_WIDTH}),
    )
    last_name = forms.CharField(
        label="Last Name",
        max_length=30,
        widget=forms.TextInput(
            attrs={'size':FORM_FIELD_WIDTH}),
    )

    # Invitation address
    invite_street1 = forms.CharField(
        label="Street Address Line 1",
        max_length=256,
        widget=forms.TextInput(
            attrs={'size':FORM_FIELD_WIDTH}),
    )

    invite_street2 = forms.CharField(
        required=False,
        label="Street Address Line 2",
        max_length=256,
        widget=forms.TextInput(
            attrs={'size':FORM_FIELD_WIDTH}),
    )

    invite_city = forms.CharField(
        label="City",
        max_length=256,
        widget=forms.TextInput(
            attrs={'size':FORM_FIELD_WIDTH}),
    )

    invite_state = forms.CharField(
        label="State/Country",
        max_length=256,
        widget=forms.TextInput(
            attrs={'size':FORM_FIELD_WIDTH}),
    )

    invite_zip = forms.CharField(
        label="Postal Code",
        max_length=256,
        widget=forms.TextInput(
            attrs={'size':FORM_FIELD_WIDTH}),
    )

    # Email address
    email_address = forms.EmailField(
        label="Email address",
        help_text="Please enter a valid email address",
        widget=forms.TextInput(
            attrs={'size':FORM_FIELD_WIDTH}),
    )

    password1 = forms.CharField(
        label="Password",
        strip=False,
        widget=forms.PasswordInput,
        help_text=password_validation.password_validators_help_text_html(),
    )
    password2 = forms.CharField(
        label="Password confirmation",
        widget=forms.PasswordInput,
        strip=False,
        help_text="Enter the same password as before, for verification.",
    )

    error_messages = {
        'username_taken':
            "Username %(username)s is not available. Please choose a different username",
        'rsvp_fail': "Sorry, we were unable to find an unclaimed RSVP under the last name %(last_name)s at address %(address)s",
        'password_mismatch': "Passwords do not match, please double-check and try again",
    }

    def get_address(self):
        return self.cleaned_data.get('invite_street1') + " " + \
            self.cleaned_data.get('invite_street2', '') + ", " + \
            self.cleaned_data.get('invite_city') + ", " + \
            self.cleaned_data.get('invite_state') + " " + \
            self.cleaned_data.get('invite_zip')

    def clean_email_address(self):
        username = self.cleaned_data.get('email_address')
        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist:
            return username
        raise forms.ValidationError(
            self.error_messages['username_taken'],
            code='username_taken',
            params={'username':username})

    def clean(self):
        """
        Ensure that all of the data is proper for the account creation
        """
        def strip_address(address):
            """
            Takes an address and returns only the numbers, in order. This is
            what we'll use to validate the RSVP claim
            """
            return re.findall('\d', address)

        password1 = self.cleaned_data.get('password1')
        password2 = self.cleaned_data.get('password2')

        # Make sure the passwords match
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(
                self.error_messages['password_mismatch'],
                code='password_mismatch',
            )

        # Make sure the address entered is mildly associated with a free
        #   address in our RSVP database
        address = self.get_address()
        last_name = self.cleaned_data.get('last_name')
        first_name = self.cleaned_data.get('first_name')
        full_name = f'{first_name} {last_name}'

        addr_nums = strip_address(address)
        if not addr_nums:
            raise forms.ValidationError(
                self.error_messages['invalid_address'],
                code='invalid_address'
            )

        # Pull all RSVPs under the given last name
        rsvps = RSVP.objects.all().filter(last_names__icontains=last_name).filter(profile=None)
        found_rsvp = None
        for rsvp in rsvps:
            if rsvp.invite_address:
                rsvp_nums = strip_address(rsvp.invite_address)
                if (rsvp_nums == addr_nums):
                    found_rsvp = rsvp
                    break
            else:
                # TODO should probably just do a join initially rather than this
                rsvp_persons = RSVPPerson.objects.all().filter(rsvp=rsvp)
                for rsvp_person in rsvp_persons:
                    # don't have an address in our records => match by full name
                    if full_name.lower() == rsvp_person.name.lower():
                        found_rsvp = rsvp
                        break

        if not found_rsvp:
            raise forms.ValidationError(
                self.error_messages['rsvp_fail'],
                code='address_fail',
                params={'address':address, 'last_name':last_name})

        # Note the RSVP
        self.rsvp = found_rsvp

        return self.cleaned_data

class RSVPForm(forms.ModelForm):
    """
    Base RSVP Form
    """
    class Meta:
        model = RSVP
        fields = ['comment']

    def __init__(self, *args, **kwargs):
        """
        Init function to make comment box optional
        """
        super(RSVPForm, self).__init__(*args, **kwargs)
        self.fields['comment'].required = False
        if not settings.WEDSITE_JSON['rsvp']['active']:
            self.fields['comment'].disabled = True

# Formset for all RSVP Persons
RSVPPersonFormSet = inlineformset_factory(RSVP, RSVPPerson,
    fields=list(settings.WEDSITE_JSON['rsvp']['fields'].keys()),
    extra=0,
    can_delete=False)






from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth.models import User
from wedsite.conf import settings
from wedsite.models import RSVP, RSVPPerson

import csv
import sys

class Command(BaseCommand):
    help = 'Imports a CSV guestlist to the RSVP table'

    def handle(self, *args, **options):
        '''Import CSV from STDIN with following fields:

        Key:    comma-separated last names that are used, along with address, to
                group RSVP people into a single RSVP
        First:  First name
        Last:   Last name
        Address:Full address, including City, State/Country, Zip

        The CSV should be comma-separated with " as the quote char, and should
        not include a header column.

        Note that this is designed to load all RSVPs from a blank slate - it
        makes no attempt to deduplicate or associate new RSVPPersons with
        existing RSVPs.

        '''
        reader = csv.reader(sys.stdin, delimiter=',')
        seen_rsvps = {}
        # TODO support a header that defines these dynamically
        field_index = {
            'Key': 0,
            'First': 1,
            'Last': 2,
            'Address': 3,
        }
        for row in reader:
            key = row[field_index['Key']]
            if key not in seen_rsvps:
                rsvp = RSVP(
                    last_names=key,
                    invite_address=row[field_index['Address']],
                )
                rsvp.save()
                seen_rsvps[key] = rsvp

            rsvp_person = RSVPPerson(
                rsvp=seen_rsvps[key],
                name='{} {}'.format(
                    row[field_index['First']],
                    row[field_index['Last']],
                ),
            )
            rsvp_person.save()



from wedsite.settings import DEFAULT_JSON
from django.conf import settings
from appconf import AppConf

class WedsiteAppConf(AppConf):
    """
    Application configuration class. Helps us to have some default settings
    for the wedsite module but allow users to override them.
    """
    JSON = DEFAULT_JSON



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.views.generic.edit import FormView, UpdateView
from django.http import HttpResponse
from django.shortcuts import render, redirect
from django.core.urlresolvers import reverse
from django.views import View
from django.utils.decorators import method_decorator
from django.contrib.auth.views import (
    redirect_to_login, SuccessURLAllowedHostsMixin)
from django.contrib.auth import (
    authenticate, login)
from django.contrib.auth.models import User
from wedsite.forms import CreateUserForm, RSVPPersonFormSet, RSVPForm
from wedsite.models import Profile, RSVP
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect
from django.views.decorators.debug import sensitive_post_parameters
from django.http import HttpResponseRedirect
from wedsite.conf import settings

class StaticView(View):
    """
    Basic static view. Shows a template back

    Will redirect to login page if the user isn't authenticated, otherwise will
    allow them to view the requested page.

    """
    template = None

    def get(self, request):
        if request.user.is_authenticated or (request.get_full_path() == reverse('index')):
            return render(request, "wedding/pages/" + self.template, {})
        else:
            return redirect_to_login(request.get_full_path())

class StaticViewNoAuth(View):
    """
    Basic static view. Shows a template back
    """
    template = None

    def get(self, request):
        return render(request, "wedding/pages/" + self.template, {})

class RSVPView(View):
    """
    RSVP View.

    Handles the form data for RSVPs.

    """
    def get_object(self, queryset=None):
        try:
            return self.request.user.profile.rsvp
        except RSVP.DoesNotExist:
            return None

    def post(self, request):
        if request.user.is_authenticated:
            rsvp = self.get_object()

            # Handle the RSVP update itself
            rsvp_form = RSVPForm(request.POST, request.FILES, instance=rsvp, prefix='rsvp')

            # Handle the RSVP Persons
            formset = RSVPPersonFormSet(request.POST, request.FILES, instance=rsvp, prefix='people')

            if formset.is_valid and rsvp_form.is_valid():
                try:
                    formset.save()
                    rsvp_form.save()
                    return HttpResponseRedirect(request.get_full_path() + '?updated=y')
                except ValueError:
                    pass



            return render(request, 'wedding/pages/rsvp.html', {'formset': formset})
        else:
            return redirect_to_login(request.get_full_path())

    def get(self, request):
        if request.user.is_authenticated:
            rsvp = self.get_object()
            updated = True if (request.GET.get('updated', '') == 'y') else False
            new_account = True if (request.GET.get('new_account', '') == 'y') else False
            formset = RSVPPersonFormSet(instance=rsvp, prefix='people') if rsvp else []

            # Need to make the formset read-only if RSVPs are already locked.
            if not settings.WEDSITE_JSON['rsvp']['active']:
                for form in formset:
                    for field in form.fields:
                        form.fields[field].disabled = True

            rsvp_form = RSVPForm(instance=rsvp, prefix='rsvp') if rsvp else None
            return render(request, 'wedding/pages/rsvp.html',
                {
                    'formset': formset,
                    'rsvp_form': rsvp_form,
                    'updated': updated,
                    'new_account': new_account
                }
            )
        else:
            return redirect_to_login(request.get_full_path())


class CreateAccountView(SuccessURLAllowedHostsMixin, FormView):
    """
    Account creation view. Will take a name, street address,
    email address and password and will create a new user if it can find
    a free RSVP that agrees with the user's info.
    """
    form_class = CreateUserForm
    success_url = "/rsvp?new_account=y"
    template_name = 'registration/create_user.html'

    @method_decorator(sensitive_post_parameters())
    @method_decorator(csrf_protect)
    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        """
        Form sending function. Note the decorators
        """
        return super(CreateAccountView, self).dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        """
        If the form is valid. Attempt to create the user, the profile, and
        link in the RSVP
        """

        # Make the database username the lowercase version of what's been
        #   entered. This will force uniqueness among the names for all
        #   new users. Checking is done in a case-insensitive manner
        username = form.cleaned_data.get('email_address').lower()

        # Finally, we want to make the user object and the profile object
        #   and link them together
        user = User.objects.create_user(
            username,
            form.cleaned_data.get('email_address'),
            form.cleaned_data.get('password1'),
            first_name=form.cleaned_data.get('first_name'),
            last_name=form.cleaned_data.get('last_name'))
        if not user:
            return HttpResponse("Failed to create user", status=400)

        # And create a profile to link to the user, also linking in the RSVP
        #   that we found
        user_profile = Profile(
            user=user,
            address=form.get_address())
        user_profile.save()
        form.rsvp.profile = user_profile
        form.rsvp.save()

        user_cache = authenticate(
            username=username,
            password=form.cleaned_data.get('password1'))
        if user_cache is None:
            return HttpResponse("Failed to log in user", status=400)

        login(self.request, user_cache)
        return HttpResponseRedirect(self.success_url)
